@@ -1,5 +1,6 @@
 import numpy as np
 from secml.array import CArray
+
 from secml_malware.attack.blackbox.c_blackbox_problem import CBlackBoxProblem
 from secml_malware.attack.blackbox.c_wrapper_phi import CWrapperPhi
 from secml_malware.utils.extend_pe import shift_section_by, shift_pe_header_by, create_int_list_from_x_adv
@@ -18,10 +19,10 @@ class CBlackBoxFormatExploitEvasionProblem(CBlackBoxProblem):
 			invalid_value: int = 256
 	):
 		super(CBlackBoxFormatExploitEvasionProblem, self).__init__(model_wrapper,
-																	latent_space_size=preferable_extension_amount + pe_header_extension,
-																	iterations=iterations,
-																	population_size=population_size, is_debug=is_debug,
-																	penalty_regularizer=penalty_regularizer)
+																   latent_space_size=preferable_extension_amount + pe_header_extension,
+																   iterations=iterations,
+																   population_size=population_size, is_debug=is_debug,
+																   penalty_regularizer=penalty_regularizer)
 		self.preferable_extension_amount = preferable_extension_amount
 		self.pe_header_extension = pe_header_extension
 		self.invalid_value = invalid_value
@@ -78,3 +79,35 @@ class CBlackBoxFormatExploitEvasionProblem(CBlackBoxProblem):
 		x_adv = CArray(x_adv)
 		x_adv = x_adv.reshape((1, x_adv.shape[-1]))
 		return CArray(x_adv)
+
+
+class CBlackBoxContentShiftingEvasionProblem(CBlackBoxFormatExploitEvasionProblem):
+	def __init__(
+			self,
+			model_wrapper: CWrapperPhi,
+			population_size: int,
+			bytes_to_inject: int = 0x200,
+			iterations: int = 100,
+			is_debug: bool = False,
+			penalty_regularizer: float = 0,
+			invalid_value: int = 256
+	):
+		super(CBlackBoxContentShiftingEvasionProblem, self).__init__(model_wrapper, population_size, bytes_to_inject, 0,
+																	 iterations, is_debug, penalty_regularizer,
+																	 invalid_value)
+
+
+class CBlackBoxContentDOSExtensionProblem(CBlackBoxFormatExploitEvasionProblem):
+	def __init__(
+			self,
+			model_wrapper: CWrapperPhi,
+			population_size: int,
+			bytes_to_inject: int = 0x200,
+			iterations: int = 100,
+			is_debug: bool = False,
+			penalty_regularizer: float = 0,
+			invalid_value: int = 256
+	):
+		super(CBlackBoxContentDOSExtensionProblem, self).__init__(model_wrapper, population_size, 0, bytes_to_inject,
+																  iterations, is_debug, penalty_regularizer,
+																  invalid_value)
