@@ -1,16 +1,14 @@
 import copy
-import datetime
+import random
 import time
+from typing import Tuple, Any
 
+import numpy as np
 from secml.adv.attacks import CAttackEvasion
 from secml.array import CArray
 from secml.data import CDataset
-from secml.optim.constraints import CConstraint
-from secml.optim.function import CFunction
-from secml.optim.optimizers import COptimizer
+
 from secml_malware.attack.blackbox.c_blackbox_problem import CBlackBoxProblem
-import random
-import numpy as np
 
 try:
 	import deap
@@ -20,8 +18,17 @@ except ImportError:
 		"Install DEAP to apply black box evasion")
 
 
-class CGeneticAlgorithm(CAttackEvasion):
+def _pad_sequence_with_last(sequence, until):
+	how_many = until - len(sequence)
+	if how_many <= 0:
+		return sequence
+	return sequence + [sequence[-1]] * how_many
 
+
+class CGeneticAlgorithm(CAttackEvasion):
+	"""
+	The genetic optimizer.
+	"""
 	def _objective_function_gradient(self, x):
 		raise NotImplementedError("This function is not used inside secML malware")
 
@@ -45,6 +52,17 @@ class CGeneticAlgorithm(CAttackEvasion):
 			problem: CBlackBoxProblem,
 			is_debug: bool = False,
 	):
+		"""
+		Create and instance of the genetic optimizer.
+
+		Parameters
+		----------
+		problem : CBlackBoxProblem
+			The problem to optimize
+		is_debug : bool, default False
+			If True, debug prints will be displayed during the attack.
+			Default is False
+		"""
 		CAttackEvasion.__init__(
 			self,
 			problem.model_wrapper.classifier,
@@ -61,8 +79,32 @@ class CGeneticAlgorithm(CAttackEvasion):
 		self.minimization_result_ = []
 		self.evolved_problem_ = None
 		self.stagnation = 5
+		self.elapsed_time_ = 0
 
-	def run(self, x, y, ds_init=None, *args, **kargs):
+	def run(self, x, y, ds_init=None) -> Tuple[CArray, CArray, CDataset, Any]:
+		"""
+		Runs the genetic algorithms.
+
+		Parameters
+		----------
+		x : CArray
+			input sample to perturb
+		y : CArray
+			original class
+		ds_init : CDataset, optional, default None
+			the initialization point.
+			Default is None
+		Returns
+		-------
+		CArray
+			y_pred : the predicted label after the attack
+		CArray
+			scores : the scores after the attack
+		CDataset
+			adv_ds : the CDataset containing the adversarial points
+		CArray
+			f_obj : the mean value for the objective function
+		"""
 		x = CArray(x).atleast_2d()
 		y = CArray(y).atleast_2d()
 		x_init = None if ds_init is None else CArray(ds_init.X).atleast_2d()
@@ -89,7 +131,7 @@ class CGeneticAlgorithm(CAttackEvasion):
 			k = idx[i].item()  # idx of sample that can be modified
 
 			xi = x[k, :] if x_init is None else x_init[k, :]
-			x_opt, f_opt = self._run(x[k, :], y[k], x_init=xi, *args, **kargs)
+			x_opt, f_opt = self._run(x[k, :], y[k], x_init=xi)
 
 			self.logger.info(
 				"Point: {:}/{:}, f(x):{:}, eval:{:}/{:}".format(k, x.shape[0], f_opt, self.f_eval, self.grad_eval))
@@ -114,36 +156,24 @@ class CGeneticAlgorithm(CAttackEvasion):
 		if x_init is None:
 			x_init = copy.deepcopy(x0)
 
-		_, current_conf = self.model_wrapper.predict(x_init, return_decision_function=True)
-		current_conf = current_conf[0, 1].item()
-		self.confidences_ = [current_conf]
-		self.sizes_ = [x_init.shape[-1]]
-		self.fitness_ = [0]
+		self._original_x = self.problem.init_starting_point(x_init)
+		current_conf = self.problem.score_step(x_init, 0)
+
 		if self.is_debug:
-			print(f'> Original Confidence: {current_conf}')
+			print(f'> Original Confidence: {self.confidences_[0]}')
 			print("> Beginning new sample evasion...")
 
-		self._original_x = self.problem.init_starting_point(x_init)
-		minimization_results = self.compute_black_box_optimization(self._original_x)
+		minimization_results = self._compute_black_box_optimization()
 
 		self.minimization_result_ = minimization_results.tondarray()
 
 		x_adv = self.problem.apply_feasible_manipulations(self.minimization_result_, self._original_x)
-		
-		_, current_conf = self.model_wrapper.predict(x_adv, return_decision_function=True)
-		current_conf = current_conf[1].item()
-		self.confidences_.append(current_conf)
+
 		if self.is_debug:
-			print(f'>AFTER INVERSION, CONFIDENCE SCORE: {current_conf}')
-		return x_adv, current_conf
+			print(f'>AFTER INVERSION, CONFIDENCE SCORE: {self.confidences_[-1]}')
+		return x_adv, self.confidences_[-1]
 
-	def pad_sequence_with_last(self, sequence, until):
-		how_many = until - len(sequence)
-		if how_many <= 0:
-			return sequence
-		return sequence + [sequence[-1]] * how_many
-
-	def compute_black_box_optimization(self, x: CArray) -> CArray:
+	def _compute_black_box_optimization(self) -> CArray:
 
 		start_t = time.time()
 
@@ -155,17 +185,17 @@ class CGeneticAlgorithm(CAttackEvasion):
 		toolbox = base.Toolbox()
 		toolbox.register('attr_float', random.random)
 		toolbox.register("individual", tools.initRepeat, creator.Individual, toolbox.attr_float,
-						 n=self.problem.latent_space_size)
+							n=self.problem.latent_space_size)
 		toolbox.register("population", tools.initRepeat, list, toolbox.individual)
 
 		toolbox.register("evaluate", self.problem.fitness)
 		toolbox.register("mate", tools.cxOnePoint)
-		toolbox.register("mutate", tools.mutPolynomialBounded, indpb=0.10, eta=1, low=0, up=1)
-		toolbox.register("select", tools.selTournament, tournsize=10)
+		toolbox.register("mutate", random_mutation, indpb=0.3)
+		toolbox.register("select", tools.selTournament, tournsize=self.problem.population_size)
 
 		pop = toolbox.population(n=self.problem.population_size)
-		fitnesses = [self.problem.fitness(np.array(t)) for t in pop]
-		for ind, fit in zip(pop, fitnesses):
+		fitness = [self.problem.fitness(np.array(t)) for t in pop]
+		for ind, fit in zip(pop, fitness):
 			ind.fitness.values = fit
 
 		# CXPB  is the probability with which two individuals are crossed
@@ -173,7 +203,7 @@ class CGeneticAlgorithm(CAttackEvasion):
 		CXPB, MUTPB = 0.9, 0.3
 		g = 0
 		last_n_best_fits = []
-		while g < self.problem.iterations: #and min(self.problem.confidences_) > self.problem.threshold:
+		while g < self.problem.iterations:
 			# Select the next generation individuals
 			offspring = toolbox.select(pop, self.problem.population_size)
 			# Clone the selected individuals
@@ -194,8 +224,8 @@ class CGeneticAlgorithm(CAttackEvasion):
 			# Evaluate the individuals with an invalid fitness
 			invalid_ind = [ind for ind in offspring if not ind.fitness.valid]
 			slice_indexes.append(len(invalid_ind))
-			fitnesses = [self.problem.fitness(np.array(t)) for t in invalid_ind]
-			for ind, fit in zip(invalid_ind, fitnesses):
+			fitness = [self.problem.fitness(np.array(t)) for t in invalid_ind]
+			for ind, fit in zip(invalid_ind, fitness):
 				ind.fitness.values = fit
 
 			pop.extend(invalid_ind)
@@ -203,17 +233,19 @@ class CGeneticAlgorithm(CAttackEvasion):
 			best_fitness = min(fits)
 			last_n_best_fits.insert(0, best_fitness)
 			last_n_best_fits = last_n_best_fits[:self.stagnation]
-			print(f'>{g} - Global min: {best_fitness}')
+			if self.is_debug:
+				print(f'>{g} - Global min: {best_fitness}')
 			if len(last_n_best_fits) == self.stagnation and all((np.array(last_n_best_fits) - best_fitness) < 1e-6):
-				print('Stagnating result!')
+				if self.is_debug:
+					print('Stagnating result!')
 				break
 			g += 1
 
-		confidences, fitness, sizes = self.problem.export_internal_results(slice_indexes)
+		confidences, fitness, sizes = self.problem._export_internal_results(slice_indexes)
 		end_t = time.time()
-		self.confidences_ = self.pad_sequence_with_last(confidences, 100)
-		self.fitness_ = self.pad_sequence_with_last(fitness, 100)
-		self.sizes_ = self.pad_sequence_with_last(sizes, 100)
+		self.confidences_ = _pad_sequence_with_last(confidences, self.problem.iterations)
+		self.fitness_ = _pad_sequence_with_last(fitness, self.problem.iterations)
+		self.sizes_ = _pad_sequence_with_last(sizes, self.problem.iterations)
 		self.elapsed_time_ = end_t - start_t
 		best_t = tools.selBest(pop, 1)[0]
 
@@ -223,8 +255,44 @@ class CGeneticAlgorithm(CAttackEvasion):
 		return CArray(best_t)
 
 
-	def write_adv_to_file(x_adv: CArray, path: str):
+
+
+	@classmethod
+	def write_adv_to_file(cls, x_adv: CArray, path: str):
+		"""
+		Write the adversarial malware as a file on disk
+
+		Parameters
+		----------
+		x_adv : CArray
+			The adversarial malware to dump
+		path : str
+			The path where to save the executable
+		"""
 		x_real = x_adv.tolist()[0]
 		x_real_adv = b''.join([bytes([i]) for i in x_real])
 		with open(path, 'wb') as f:
 			f.write(x_real_adv)
+
+def random_mutation(individual, indpb):
+	"""
+	Apply the mutation operator, that perturb randomly each entry of the individual, with a given probability.
+	The mutation is applied in-place.
+
+	Parameters
+	----------
+	individual :
+		the individual to mutate
+	indpb : float
+		the probability of altering a single entry
+	Returns
+	-------
+	tuple
+		the mutated individual, the mutatio is in-place
+	"""
+	size = len(individual)
+	for i in range(size):
+		if random.random() < indpb:
+			individual[i] = random.random()
+	return individual,
+
