@@ -71,7 +71,7 @@ class CFastGradientSignMethodEvasion(CEnd2EndMalwareEvasion):
 		return confidence.item()
 
 	def invert_feature_mapping(self, x, x_adv):
-		E = self.get_embedded_byte_matrix()
+		E = self._get_embedded_byte_matrix()
 		byte_malware = torch.zeros(x.shape[-1]) + self.classifier.get_embedding_value()
 		E[self.invalid_pos] = torch.tensor([np.infty for _ in range(self.classifier.get_embedding_size())])
 		reconstructed_x = x.tondarray()
